# -*- coding: utf-8 -*-
#    Tichy
#    copyright 2008 Guillaume Chereau (charlie137@gmail.com)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.

import random
import codecs

import logging
logger = logging.getLogger('app.learn')

import tichy
import tichy.gui as gui
import tichy.style

from brain import Brain
from dic import Dic


class Choice(tichy.Item):

    def __init__(self, value):
        self.value = value

    def view(self, parent):
        # XXX: Horrible hack !
        ret = gui.Button(parent, min_size=gui.Vect(160, 100), expand=True)
        gui.Label(ret, self.value)

        def on_clicked(b):
            self.emit('selected')
        ret.connect('clicked', on_clicked)

        return ret


class ChoicesView(gui.Table):

    def __init__(self, parent, choice, item=None, **kargs):
        super(ChoicesView, self).__init__(parent, axis=1, item=choice, **kargs)
        self.item.connect('modified', self.redraw)
        self.redraw()

    def redraw(self, *args):
        for c in self.children[:]:
            c.destroy()
        for c in self.item:
            c.view(self)


class Answer(tichy.Application):

    def run(self, window, card, correct):
        self.name = "Correct" if correct else "Wrong"
        frame = self.view(window)

        vbox = gui.Box(frame, axis=1)
        gui.Label(vbox, card.q, font_size=58, expand=True)
        gui.Label(vbox, card.a)
        if card.comment:
            gui.Label(vbox, "(%s)" % card.comment)

        gui.Spring(vbox, axis=1)

        read_button = gui.Button(vbox)
        gui.Label(read_button, "Read")
        read_button.connect('clicked', self.on_read, card.a)

        hbox = gui.Box(vbox, axis=0, border=0, expand=True)

        next_button = gui.Button(hbox, expand=True)
        gui.Label(next_button, "OK")

        if correct:
            relearn_button = gui.Button(hbox)
            gui.Label(relearn_button, "Re-learn")

            choice, value = yield tichy.tasklet.WaitFirst(
                tichy.tasklet.Wait(next_button, 'clicked'),
                tichy.tasklet.Wait(relearn_button, 'clicked'))
            yield choice == 0
        else:
            yield tichy.tasklet.Wait(next_button, 'clicked')
            yield False

    def on_read(self, b, msg):
        speak = tichy.SpeakService.get()
        speak.speak(msg, lang='zh')


class Learn(tichy.Application):

    name = 'Learn'
    category = 'main/chinese'
    icon = 'icon.png'

    def run(self, window):
        self.window = window
        frame = self.view(window, back_button=True)

        vbox = gui.Box(frame, axis=1, expand=True)
        self.question_text = tichy.Text("Question")
        self.question_text.view(vbox, expand=True, font_size=58)

        self.choices = tichy.List()
        ChoicesView(vbox, self.choices, expand=True)

        frame.connect('back', self.on_quit)

        dic = codecs.open(Learn.path('characters.dic'), encoding='utf-8')

        logger.info("opening the dict")
        dic = Dic.read(dic)
        self.full_dic = dic[:]
        try:
            logger.info("read brain")
            path = tichy.Persistance('learn/characters.brn').path
            file = codecs.open(path, encoding='utf-8')
            brain = Brain.read(file, dic)
        except Exception, ex:
            logger.error("can't read brain : %s", ex)
            brain = Brain()

        logger.info("start game")
        self.task = None
        while True:
            self.task = brain.ask(self.ask, dic)
            try:
                yield self.task
            except GeneratorExit:
                break
        logger.info("Save brain")
        path = tichy.Persistance('learn/characters.brn').path
        file = codecs.open(path, 'w', encoding='utf-8')
        brain.write(file)

    def on_quit(self, *args):
        self.task.exit()

    @tichy.tasklet.tasklet
    def ask(self, card, level):
        logger.info("ask")
        question = card.q

        logger.info("generate answers set")
        # We also use 8 random value from the dict
        rand_answers = [random.choice(self.full_dic).a for i in range(8)]
        answers = rand_answers + [card.a]
        random.shuffle(answers)

        self.question_text.value = question
        self.choices.clear()

        choices = [Choice(a) for a in answers]

        self.choices[:] = choices

        logger.info("wait for answer")
        ret = yield tichy.WaitFirst(
            *[tichy.Wait(c, 'selected') for c in choices])
        ret = answers[ret[0]]

        ret = yield Answer(self.window, card, ret==card.a)
        yield ret
