#    Tichy
#
#    copyright 2008 Guillaume Chereau (charlie137@gmail.com)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful, but
#    WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.


"""Persistance module"""

# TODO: turn the save and load methods into a `Tasklet`. Saving into a
#       file can take a lot of time.

import os
import yaml

import logging
LOGGER = logging.getLogger('persistance')


class Persistance(object):
    """Use this class to save and load data from file

    All the data will be placed into ~/.tichy directory. They are
    stored using yaml format, but we could modify this cause the
    plugins are not suppoed to read the file directly.
    """

    base_path = os.path.expanduser('~/.tichy/')

    def __init__(self, file_):
        self.file = file_

        dir_ = os.path.dirname(self.path)
        if not os.path.exists(dir_):
            os.makedirs(dir_)

    @property
    def path(self):
        return os.path.join(self.base_path, self.file)

    def open(self, mod='r'):
        return open(self.path, mod)

    def save(self, data):
        """Save a data into the file

        :Parameters:

            data
                Any kind of python structure that can be
                serialized. Usually dictionary or list.
        """
        file = self.open('w')
        file.write(yaml.safe_dump(data, default_flow_style=False))

    def load(self):
        """Load data from the file

        :Returns: The structure previously saved into the file
        """
        file = self.open()
        return yaml.safe_load(file)
